@@ -61,9 +61,6 @@ pub struct Match {
 /// Information about a placeholder bound in a match.
 #[derive(Debug)]
 pub(crate) struct PlaceholderMatch {
-    /// The node that the placeholder matched to. If set, then we'll search for further matches
-    /// within this node. It isn't set when we match tokens within a macro call's token tree.
-    pub(crate) node: Option<SyntaxNode>,
     pub(crate) range: FileRange,
     /// More matches, found within `node`.
     pub(crate) inner_matches: SsrMatches,
@@ -186,7 +183,7 @@ impl<'db, 'sema> Matcher<'db, 'sema> {
                 self.validate_range(&original_range)?;
                 matches_out.placeholder_values.insert(
                     placeholder.ident.clone(),
-                    PlaceholderMatch::new(Some(code), original_range),
+                    PlaceholderMatch::from_range(original_range),
                 );
             }
             return Ok(());
@@ -465,7 +462,7 @@ impl<'db, 'sema> Matcher<'db, 'sema> {
                 let mut last_matched_token = child;
                 // Read code tokens util we reach one equal to the next token from our pattern
                 // or we reach the end of the token tree.
-                while let Some(next) = children.next() {
+                for next in &mut children {
                     match &next {
                         SyntaxElement::Token(t) => {
                             if Some(t.to_string()) == next_pattern_token {
@@ -530,7 +527,7 @@ impl<'db, 'sema> Matcher<'db, 'sema> {
         pattern_ufcs: &UfcsCallInfo,
         code: &ast::MethodCallExpr,
     ) -> Result<(), MatchFailed> {
-        use ast::ArgListOwner;
+        use ast::HasArgList;
         let code_resolved_function = self
             .sema
             .resolve_method_call(code)
@@ -590,7 +587,7 @@ impl<'db, 'sema> Matcher<'db, 'sema> {
         pattern_ufcs: &UfcsCallInfo,
         code: &ast::CallExpr,
     ) -> Result<(), MatchFailed> {
-        use ast::ArgListOwner;
+        use ast::HasArgList;
         // Check that the first argument is the expected type.
         if let (Some(pattern_type), Some(expr)) = (
             &pattern_ufcs.qualifier_type,
@@ -609,9 +606,13 @@ impl<'db, 'sema> Matcher<'db, 'sema> {
         expr: &ast::Expr,
     ) -> Result<usize, MatchFailed> {
         use hir::HirDisplay;
-        let code_type = self.sema.type_of_expr(expr).ok_or_else(|| {
-            match_error!("Failed to get receiver type for `{}`", expr.syntax().text())
-        })?;
+        let code_type = self
+            .sema
+            .type_of_expr(expr)
+            .ok_or_else(|| {
+                match_error!("Failed to get receiver type for `{}`", expr.syntax().text())
+            })?
+            .original;
         // Temporary needed to make the borrow checker happy.
         let res = code_type
             .autoderef(self.sema.db)
@@ -645,8 +646,8 @@ impl Match {
     ) -> Result<(), MatchFailed> {
         let module = sema
             .scope(&self.matched_node)
-            .module()
-            .ok_or_else(|| match_error!("Matched node isn't in a module"))?;
+            .ok_or_else(|| match_error!("Matched node isn't in a module"))?
+            .module();
         for (path, resolved_path) in &template.resolved_paths {
             if let hir::PathResolution::Def(module_def) = resolved_path.resolution {
                 let mod_path = module.find_use_path(sema.db, module_def).ok_or_else(|| {
@@ -711,19 +712,14 @@ fn recording_match_fail_reasons() -> bool {
 }
 
 impl PlaceholderMatch {
-    fn new(node: Option<&SyntaxNode>, range: FileRange) -> Self {
+    fn from_range(range: FileRange) -> Self {
         Self {
-            node: node.cloned(),
             range,
             inner_matches: SsrMatches::default(),
             autoderef_count: 0,
             autoref_kind: ast::SelfParamKind::Owned,
         }
     }
-
-    fn from_range(range: FileRange) -> Self {
-        Self::new(None, range)
-    }
 }
 
 impl NodeKind {
@@ -767,7 +763,7 @@ impl Iterator for PatternIterator {
     type Item = SyntaxElement;
 
     fn next(&mut self) -> Option<SyntaxElement> {
-        while let Some(element) = self.iter.next() {
+        for element in &mut self.iter {
             if !element.kind().is_trivia() {
                 return Some(element);
             }
@@ -784,7 +780,6 @@ impl PatternIterator {
 
 #[cfg(test)]
 mod tests {
-    use super::*;
     use crate::{MatchFinder, SsrRule};
 
     #[test]
@@ -793,20 +788,12 @@ mod tests {
         let input = "fn foo() {} fn bar() {} fn main() { foo(1+2); }";
 
         let (db, position, selections) = crate::tests::single_file(input);
-        let mut match_finder = MatchFinder::in_context(&db, position, selections);
+        let mut match_finder = MatchFinder::in_context(&db, position, selections).unwrap();
         match_finder.add_rule(rule).unwrap();
         let matches = match_finder.matches();
         assert_eq!(matches.matches.len(), 1);
         assert_eq!(matches.matches[0].matched_node.text(), "foo(1+2)");
         assert_eq!(matches.matches[0].placeholder_values.len(), 1);
-        assert_eq!(
-            matches.matches[0].placeholder_values[&Var("x".to_string())]
-                .node
-                .as_ref()
-                .unwrap()
-                .text(),
-            "1+2"
-        );
 
         let edits = match_finder.edits();
         assert_eq!(edits.len(), 1);
